package com.persagy.dmp.alarm.jms;

import cn.hutool.core.thread.ExecutorBuilder;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 消费消息线程池
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class ConsumeMessageThreadPool {
    private final ExecutorService es;

    @Autowired
    private MessageProcesser messageProcesser;

    public ConsumeMessageThreadPool() {
        es = ExecutorBuilder.create()
                .setCorePoolSize(5)
                .setMaxPoolSize(10)
                .setWorkQueue(new LinkedBlockingQueue<>(10))
                .setHandler(new ThreadPoolExecutor.CallerRunsPolicy())
                .build();
    }

    public void consumeMessage(DmpMessage message, Channel channel, Message msg) {
        es.execute(() -> {
            try {
                log.info("============================== Receive:" + JacksonMapper.toSimpleJson(message));
                messageProcesser.listen(message);
                // 手动确认消息已消费
                channel.basicAck(msg.getMessageProperties().getDeliveryTag(), false);
            } catch (Exception e) {
                log.error(e.getMessage(),e);
                Boolean isRedeliveredFail = msg.getMessageProperties().getRedelivered();
                // true表示消息已经重复处理失败
                try {
                    if (isRedeliveredFail) {
                        // 拒绝消息，requeue=false 表示不再重新入队，如果配置了死信队列则进入死信队列
                        log.error("重复消费消息失败，message: {}", JacksonMapper.toSimpleJson(message));
                        channel.basicReject(msg.getMessageProperties().getDeliveryTag(), false);
                    } else {
                        // 如果是第一次失败则再次放入队列
                        // requeue为是否重新回到队列，true重新入队
                        channel.basicNack(msg.getMessageProperties().getDeliveryTag(), false, true);
                    }
                } catch (Exception exception) {
                    log.error("重复消费消息失败", exception);
                }
            }
        });
    }
}
